# -*- coding: utf-8 -*-

# Generated by Django 1.11 on 2017-06-19 16:48


from django.db import migrations
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0011_user_permission_mapping'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='organization',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Organization', verbose_name='Organiza\xe7\xe3o'),
        ),
        migrations.AddField(
            model_name='user',
            name='unit',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Unit', verbose_name='Unidade'),
        ),
    ]
