{% if self.as_pdf %}{{ self.percentual }}%{% else %}
<div class="progress" data-toggle="tooltip" data-placement="top" title="" data-original-title="{{ self.percentual }}"><div class="progress-bar" style="width: {{ self.percentual }}%;"></div></div>
{% endif %}