{% if objects %}
<table class="table table-striped table-bordered table-hover">
    <thead>
        <tr>
            <th style="text-align: center; width: 15px">{{ widget }}</th>
			<th>Opções</th>
        </tr>
    </thead>
    <tbody>
    {% for obj in objects %}
		<tr>
			<td align="center">{{ obj.widget }}</td>
			<td align="left">{{ obj|format }}</td>
		</tr>
	{% endfor %}
	</tbody>
</table>
{% endif %}
